
#
# auto-generated by remakemigrations.py
#
# ! Please make sure to apply any changes to the template in that script !
#
from django.db import migrations

from lofar.sas.tmss.tmss.tmssapp.populate import *

class Migration(migrations.Migration):

    dependencies = [
        ('tmssapp', '0001_initial'),
    ]

    operations = [ migrations.RunSQL('ALTER SEQUENCE tmssapp_SubTask_id_seq RESTART WITH 2000000;'), # Start SubTask id with 2 000 000 to avoid overlap with 'old' (test/production) OTDB
                   # add an SQL trigger in the database enforcing correct state transitions.
                   # it is crucial that illegal subtask state transitions are block at the "lowest level" (i.e.: in the database) so we can guarantee that the subtask state machine never breaks.
                   # see: https://support.astron.nl/confluence/display/TMSS/Subtask+State+Machine
                   # Explanation of SQl below: A trigger function is called upon each create/update of the subtask.
                   # If the state changes, then it is checked if the state transition from old to new is present in the SubtaskAllowedStateTransitions table.
                   # If not an Exception is raised, thus enforcing a rollback, thus enforcing the state machine to follow the design.
                   # It is thereby enforced upon the user/caller to handle these blocked illegal state transitions, and act more wisely.
                   migrations.RunSQL('''CREATE OR REPLACE FUNCTION tmssapp_check_subtask_state_transition()
                                     RETURNS trigger AS
                                     $BODY$
                                     BEGIN
                                       IF TG_OP = 'INSERT' THEN
                                         IF NOT (SELECT EXISTS(SELECT id FROM tmssapp_subtaskallowedstatetransitions WHERE old_state_id IS NULL AND new_state_id=NEW.state_id)) THEN
                                            RAISE EXCEPTION 'ILLEGAL SUBTASK STATE TRANSITION FROM % TO %', NULL, NEW.state_id;
                                         END IF;
                                       END IF;
                                       IF TG_OP = 'UPDATE' THEN
                                         IF OLD.state_id <> NEW.state_id AND NOT (SELECT EXISTS(SELECT id FROM tmssapp_subtaskallowedstatetransitions WHERE old_state_id=OLD.state_id AND new_state_id=NEW.state_id)) THEN
                                           RAISE EXCEPTION 'ILLEGAL SUBTASK STATE TRANSITION FROM "%" TO "%"', OLD.state_id, NEW.state_id;
                                         END IF;
                                       END IF;
                                     RETURN NEW;
                                     END;
                                     $BODY$
                                     LANGUAGE plpgsql VOLATILE;
                                     DROP TRIGGER IF EXISTS tmssapp_trigger_on_check_subtask_state_transition ON tmssapp_SubTask ;
                                     CREATE TRIGGER tmssapp_trigger_on_check_subtask_state_transition
                                     BEFORE INSERT OR UPDATE ON tmssapp_SubTask
                                     FOR EACH ROW EXECUTE PROCEDURE tmssapp_check_subtask_state_transition();'''),
                   migrations.RunPython(populate_choices),
                   migrations.RunPython(populate_subtask_allowed_state_transitions),
                   migrations.RunPython(populate_settings),
                   migrations.RunPython(populate_misc),
                   migrations.RunPython(populate_resources),
                   migrations.RunPython(populate_cycles),
                   migrations.RunPython(populate_projects) ]

